import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Employer } from '../../../models/employer.model';
import { NotificationService } from '../../../services/notification.service';
import { AdminService } from '../../services/admin.service';
import { AttachedUsersComponent } from '../manage-jobs/attached-users/attached-users.component';
import { EmployerRenderer } from '../manage-users/employer-renderer/employer-renderer.component';
import { DeleteEmployerComponent } from './delete-employer/delete-employer.component';
import { ValidateEmployerComponent } from './validate-employer/validate-employer.component';

@Component({
  selector: 'app-admin-manage-employers',
  templateUrl: './manage-employers.component.html',
  styleUrls: ['./manage-employers.component.scss'],
})
export class ManageEmployersComponent implements OnInit {
  public newEmployerForm: FormGroup;
  public validatedEmployers: Employer[] = [];
  public unvalidatedEmployers: Employer[] = [];
  public deleteModalOpenned = false;
  public validatedEmployersName: String[] = [];
  public mergeEmployerModalOpenned = false;
  public employerToDelete: Employer = null;
  public mergeOpe: any = {
    newEmployer: {
      name: null,
      validated: false,
    },
    formerId: '',
  };
  public contextRow: any;

  public columnDefs;
  public unvalidEmployersColumnDefs;
  public frameworkComponents;
  public defaultColDef = {
    editable: true,
    sortable: true,
    flex: 1,
    minWidth: 100,
    filter: true,
    resizable: true,
  };
  public unvalidColDef = {
    ...this.defaultColDef,
    cellStyle: {
      color: '#da6c2e',
    },
  };

  constructor(
    private adminService: AdminService,
    private notificationService: NotificationService,
    private formBuilder: FormBuilder
  ) {
    this.findValidatedEmployers();
    this.findUnvalidatedEmployers();
    this.unvalidEmployersColumnDefs = [
      {
        headerName: 'Employeur',
        field: 'name',
        editable: true,
        onCellValueChanged: this.onChange.bind(this),
      },
      {
        headerName: 'Acteurs rattachés',
        cellRenderer: 'attachedUsersComponent',
        cellRendererParams: {
          users: 'users',
        },
        minWidth: 500,
      },
      {
        headerName: 'Fusionner',
        field: '_id',
        cellRenderer: 'employerRenderer',
        singleClickEdit: true,
        cellEditor: 'agSelectCellEditor',
        cellEditorParams: {
          values: this.validatedEmployersName,
        },
        onCellValueChanged: this.onMerge.bind(this),
      },
      {
        headerName: 'Valider',
        editable: false,
        minWidth: 100,
        cellRenderer: 'validateEmployerComponent',
        cellRendererParams: {
          onClick: this.onValidateButtonClick.bind(this),
          label: 'Valider',
        },
      },
    ];
    this.columnDefs = [
      {
        headerName: 'Employeur',
        field: 'name',
        editable: true,
        onCellValueChanged: this.onChange.bind(this),
      },
      {
        headerName: 'Acteurs rattachés',
        cellRenderer: 'attachedUsersComponent',
        cellRendererParams: {
          users: 'users',
        },
        minWidth: 500,
      },
      {
        headerName: 'Suppression',
        editable: false,
        minWidth: 100,
        cellRenderer: 'deleteEmployerComponent',
        cellRendererParams: {
          onClick: this.onDeleteButtonClick.bind(this),
          label: 'Supprimer',
        },
      },
    ];
    this.frameworkComponents = {
      employerRenderer: EmployerRenderer,
      deleteEmployerComponent: DeleteEmployerComponent,
      attachedUsersComponent: AttachedUsersComponent,
      validateEmployerComponent: ValidateEmployerComponent,
    };
  }

  ngOnInit(): void {
    this.newEmployerForm = this.formBuilder.group({
      employer: ['', Validators.required],
    });
  }

  public onDeleteButtonClick(arg): void {
    this.deleteEmployer(arg.data, false);
  }

  public onSubmit(): void {
    if (!this.newEmployerForm.valid) {
      return;
    }
    const name: string = this.newEmployerForm.value.employer;
    this.adminService.createEmployer(name).subscribe(
      () => {
        this.findValidatedEmployers();
        this.newEmployerForm.reset();
        this.notificationService.showSuccess('Votre employeur a bien été créée', '');
      },
      (err) => {
        this.notificationService.showError(`${err.error.message}`, 'Une erreur est survenue');
      }
    );
  }

  public onValidateButtonClick(arg): void {
    this.validateEmployer(arg.data._id);
  }

  public deleteEmployer(employer: Employer, shouldDelete: boolean): void {
    this.toggleDeleteEmployerModal(employer);
    if (shouldDelete) {
      this.adminService.deleteEmployer(employer._id).subscribe(
        (data) => {
          this.validatedEmployers = this.validatedEmployers.filter((obj) => obj._id !== data._id);
          this.notificationService.showSuccess("L'employeur a bien été supprimée.", '');
        },
        (err) => {
          this.notificationService.showError(`${err.error.message}`, 'Une erreur est survenue');
        }
      );
    }
  }

  public onChange(arg): void {
    this.adminService.editEmployer(arg.data._id, arg.data.name).subscribe(() => {
      this.notificationService.showSuccess("L'opération a réussie.", '');
      this.findValidatedEmployers();
      this.findUnvalidatedEmployers();
    });
  }

  public onMerge(arg): void {
    const selectedEmployer = this.validatedEmployers.find((employer) => employer.name === arg.newValue);
    if (arg.oldValue === undefined) {
      return;
    }
    if (selectedEmployer === arg.oldValue) {
      return;
    }
    this.toggleMergeEmployerModal(selectedEmployer, arg.oldValue, arg);
  }

  public mergeEmployer(newEmployerId: string, formerId: string, shouldMerge: boolean, context?): void {
    this.mergeEmployerModalOpenned = !this.mergeEmployerModalOpenned;
    if (shouldMerge) {
      this.adminService.mergeEmployer(formerId, newEmployerId).subscribe((data) => {
        context.node.data[context.colDef.field] = data;
        context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
        this.findValidatedEmployers();
        this.findUnvalidatedEmployers();
        this.notificationService.showSuccess("L'opération a réussie.", '');
      });
    } else {
      if (context.oldValue) {
        context.node.data[context.colDef.field] = context.oldValue;
        context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
      }
    }
  }

  public validateEmployer(employerId: string, context?): void {
    this.adminService.validateEmployer(employerId).subscribe((data) => {
      this.findValidatedEmployers();
      this.findUnvalidatedEmployers();
    });
  }

  public toggleDeleteEmployerModal(employerToDelete: Employer): void {
    this.employerToDelete = employerToDelete;
    this.deleteModalOpenned = !this.deleteModalOpenned;
  }

  public toggleMergeEmployerModal(employerToMerge: Employer, formerId: string, dataGrid): void {
    this.mergeOpe.formerId = formerId;
    this.mergeOpe.newEmployer = employerToMerge;
    this.contextRow = dataGrid;
    this.mergeEmployerModalOpenned = !this.mergeEmployerModalOpenned;
  }

  public findValidatedEmployers(): void {
    this.validatedEmployersName = [];
    this.adminService.getValidatedEmployers().subscribe((employers) => {
      this.validatedEmployers = employers;
      employers.map((employer) => {
        this.validatedEmployersName.push(employer.name);
      });
    });
  }

  public findUnvalidatedEmployers(): void {
    this.adminService.getUnvalidatedEmployers().subscribe((employers) => {
      this.unvalidatedEmployers = employers;
    });
  }

  public getRowHeight(params): number {
    return params.data.users ? (params.data.users.length != 0 ? params.data.users.length * 40 : 40) : 40;
  }
}
