import { FormArray, FormControl, FormGroup, Validators } from '@angular/forms';
import { structureFormStep } from '../form/form-view/structure-form/structureFormStep.enum';
import { Day } from '../models/day.model';
import { Time } from '../models/time.model';
import { Structure } from '../models/structure.model';
import { CustomRegExp } from './CustomRegExp';

export interface IStructureSummary {
  step: structureFormStep;
  name: string;
  inputs: Array<string> | null;
  hideForEdit?: boolean;
}
export class formUtils {
  public structureSummary: Array<IStructureSummary> = [
    { step: structureFormStep.structureNameAndAddress, name: 'Nom et adresse', inputs: ['structureName', 'address'] },
    { step: structureFormStep.structureContact, name: 'Téléphone', inputs: ['contactMail', 'contactPhone'] },
    { step: structureFormStep.structureType, name: 'Type de structure', inputs: ['structureType'] },
    { step: structureFormStep.structureAccessModality, name: "Modalités d'accueil", inputs: ['accessModality'] },
    { step: structureFormStep.structureHours, name: "Horaires d'ouverture", inputs: ['exceptionalClosures'] },
    {
      step: structureFormStep.structurePmr,
      name: 'Accessibilité pour les personnes à mobilité réduite',
      inputs: ['pmrAccess'],
    },
    {
      step: structureFormStep.structureWebAndSocialNetwork,
      name: 'Présence sur internet',
      inputs: ['website', 'facebook', 'twitter', 'instagram'],
    },
    { step: structureFormStep.structurePublicTarget, name: 'Public admis', inputs: ['publics'] },

    {
      step: structureFormStep.structureDigitalHelpingAccompaniment,
      name: 'Aides au numérique',
      inputs: ['proceduresAccompaniment'],
    },
    { step: structureFormStep.structureTrainingType, name: 'Ateliers au numérique proposés', inputs: null },
    { step: structureFormStep.structureTrainingPrice, name: 'Gratuité des ateliers', inputs: ['freeWorkShop'] },
    { step: structureFormStep.structureWifi, name: 'Wifi', inputs: ['equipmentsAndServices'] },
    { step: structureFormStep.structureEquipments, name: 'Equipements mis à disposition', inputs: null },
    { step: structureFormStep.structureLabels, name: 'Labélisations proposées', inputs: null },
    {
      step: structureFormStep.structureOtherServices,
      name: 'Autres services proposés',
      inputs: ['equipmentsAndServices'],
    },
    { step: structureFormStep.structureDescription, name: 'Présentation de la structure', inputs: null },
    { step: structureFormStep.structureCovidInfo, name: 'Informations spécifiques à la période COVID', inputs: null },
    {
      step: structureFormStep.structureConsent,
      name: 'Partage de données sur data.grandlyon.com',
      inputs: ['dataShareConsentDate'],
    },
    {
      step: structureFormStep.structureChoiceCompletion,
      name: 'Completion info structure',
      inputs: ['choiceCompletion'],
      hideForEdit: true,
    },
    {
      step: structureFormStep.structureContactCompletion,
      name: 'Personne contact informations',
      inputs: ['contactPersonFirstname', 'contactPersonLastname', 'contactPersonEmail'],
      hideForEdit: true,
    },
    {
      step: structureFormStep.structureAccompanimentChoice,
      name: "Lieu d'accueil",
      inputs: ['placeOfReception'],
      hideForEdit: true,
    },
  ];
  public createStructureForm(structure: Structure, isEditMode?: boolean): FormGroup {
    return new FormGroup({
      _id: new FormControl(structure._id),
      coord: new FormControl(structure.coord),
      structureType: new FormControl(structure.structureType, Validators.required),
      structureName: new FormControl(structure.structureName, Validators.required),
      description: new FormControl(structure.description),
      lockdownActivity: new FormControl(structure.lockdownActivity),
      address: new FormGroup({
        numero: new FormControl(structure.address.numero),
        street: new FormControl(structure.address.street, Validators.required),
        commune: new FormControl(structure.address.commune, Validators.required),
      }),
      contactMail: new FormControl(structure.contactMail === 'unknown@unknown.com' ? null : structure.contactMail, [
        Validators.required,
        Validators.pattern(CustomRegExp.EMAIL),
      ]),
      contactPhone: new FormControl(structure.contactPhone, [
        Validators.required,
        Validators.pattern(CustomRegExp.PHONE),
      ]),
      contactPersonFirstname: new FormControl(structure.contactPersonLastName, !isEditMode && Validators.required),
      contactPersonLastname: new FormControl(structure.contactPersonLastName, !isEditMode && Validators.required),
      contactPersonEmail: new FormControl(
        structure.contactPersonEmail,
        !isEditMode && [Validators.pattern(CustomRegExp.EMAIL), Validators.required]
      ),
      website: new FormControl(structure.website, Validators.pattern(CustomRegExp.WEBSITE)),
      facebook: new FormControl(structure.facebook, Validators.pattern(CustomRegExp.FACEBOOK)),
      twitter: new FormControl(structure.twitter, Validators.pattern(CustomRegExp.TWITTER)),
      instagram: new FormControl(structure.instagram, Validators.pattern(CustomRegExp.INSTAGRAM)),
      linkedin: new FormControl(structure.linkedin, Validators.pattern(CustomRegExp.LINKEDIN)),
      hours: new FormGroup({}),
      pmrAccess: new FormControl(structure.pmrAccess, Validators.required),
      placeOfReception: new FormControl(structure.placeOfReception, !isEditMode && Validators.required),
      choiceCompletion: new FormControl(structure.choiceCompletion, !isEditMode && Validators.required),
      exceptionalClosures: new FormControl(structure.exceptionalClosures),
      labelsQualifications: this.loadArrayForCheckbox(structure.labelsQualifications, false),
      accessModality: this.loadArrayForCheckbox(structure.accessModality, true),
      publicsAccompaniment: this.loadArrayForCheckbox(structure.publicsAccompaniment, false),
      proceduresAccompaniment: this.loadArrayForCheckbox(structure.proceduresAccompaniment, false),
      //TODO: remettre ou migrer les données de accompagnements à distance
      remoteAccompaniment: new FormControl(false),
      otherDescription: new FormControl(structure.otherDescription),
      equipmentsAndServices: this.loadArrayForCheckbox(structure.equipmentsAndServices, false),
      publics: this.loadArrayForCheckbox(structure.publics, true),
      baseSkills: new FormControl(structure.baseSkills),
      accessRight: new FormControl(structure.accessRight),
      parentingHelp: new FormControl(structure.parentingHelp),
      socialAndProfessional: new FormControl(structure.socialAndProfessional),
      digitalCultureSecurity: new FormControl(structure.digitalCultureSecurity),
      nbComputers: new FormControl(
        structure.equipmentsAndServices.includes('ordinateurs') ? structure.nbComputers : 0,
        [
          Validators.required,
          Validators.pattern(CustomRegExp.NO_NEGATIVE_NUMBER),
          Validators.min(0),
          Validators.max(1000),
        ]
      ),
      nbPrinters: new FormControl(structure.equipmentsAndServices.includes('imprimantes') ? structure.nbPrinters : 0, [
        Validators.required,
        Validators.pattern(CustomRegExp.NO_NEGATIVE_NUMBER),
        Validators.min(0),
        Validators.max(1000),
      ]),
      nbTablets: new FormControl(structure.equipmentsAndServices.includes('tablettes') ? structure.nbTablets : 0, [
        Validators.required,
        Validators.pattern(CustomRegExp.NO_NEGATIVE_NUMBER),
        Validators.min(0),
        Validators.max(1000),
      ]),
      nbNumericTerminal: new FormControl(
        structure.equipmentsAndServices.includes('bornesNumeriques') ? structure.nbNumericTerminal : 0,
        [
          Validators.required,
          Validators.pattern(CustomRegExp.NO_NEGATIVE_NUMBER),
          Validators.min(0),
          Validators.max(1000),
        ]
      ),
      nbScanners: new FormControl(structure.equipmentsAndServices.includes('scanners') ? structure.nbScanners : 0, [
        Validators.required,
        Validators.pattern(CustomRegExp.NO_NEGATIVE_NUMBER),
        Validators.min(0),
        Validators.max(1000),
      ]),
      freeWorkShop: new FormControl(structure.freeWorkShop, [Validators.required]),
      dataShareConsentDate: new FormControl(structure.dataShareConsentDate),
      personalOffers: new FormControl(structure.personalOffers),
    });
  }

  public loadArrayForCheckbox(array: string[], isRequired: boolean): FormArray {
    return new FormArray(
      array.map((str) => new FormControl(str)),
      isRequired ? Validators.required : Validators.nullValidator
    );
  }

  public createDay(day: Day): FormGroup {
    return new FormGroup({
      open: new FormControl(day.open, Validators.required),
      time: new FormArray(day.time.map((oneTime) => this.createTime(oneTime))) as FormArray,
    });
  }
  public createTime(time: Time): FormGroup {
    return new FormGroup({
      opening: new FormControl(time.opening),
      closing: new FormControl(time.closing),
    });
  }
  public createHoursForm(structure: Structure): FormGroup {
    return new FormGroup({
      monday: this.createDay(structure.hours.monday),
      tuesday: this.createDay(structure.hours.tuesday),
      wednesday: this.createDay(structure.hours.wednesday),
      thursday: this.createDay(structure.hours.thursday),
      friday: this.createDay(structure.hours.friday),
      saturday: this.createDay(structure.hours.saturday),
      sunday: this.createDay(structure.hours.sunday),
    });
  }
}
